package F5::GlobalSettings;

use strict;
use warnings;
use JSON;
use File::Slurp qw(read_file);

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;
    my $self = bless({ %params }, $class);
    $self->init();
    return $self;
}

#----------------------------------------------------------------------

sub init {
    my $self = shift;
    if ($self->{conf_file} && $self->{conf_file}->{fcdn_path}) {
        $self->load_from_file();
    }
    $self->set_values();
}

#----------------------------------------------------------------------

sub apply {
    my($self, %params) = @_;
    my $rv = {};
    if ($self->{seed} && $self->{digest}) {
        my $msg = "Both seed and digest are configured";
        $self->{logger}->error($msg) if $self->{logger};
        $rv->{completed_successfully} = 0;
        $rv->{failure_msg} = $msg;
    }
    elsif ($self->{seed}) {
        $self->{logger}->info("Data protection cookie configuration=$self->{seed}")  if $self->{logger};
        $self->{logger}->info("Generating new data protection cookie") if $self->{logger};
        $rv = $self->{client}->update_data_protection($self->{client}->{session},
                                                      F5::ASMConfig::CallFlags->new(),
                                                      undef,
                                                      F5::ASMConfig::DataProtection->new(secure_key => F5::DataProtection::generate_key($self->{seed}))
                                                     );
        $rv->{configured_settings} = $self->{seed};
    }
    elsif ($self->{digest}) {
        $self->{logger}->info("Data protection cookie configuration=$self->{digest}")  if $self->{logger};
        $self->{logger}->info("Generating new data protection cookie") if $self->{logger};
        eval {
            $rv = $self->{client}->import_data_protection($self->{client}->{session},
                                                          F5::ASMConfig::CallFlags->new(),
                                                          undef,
                                                          F5::ASMConfig::DataProtectionImportExport->new(import_text => $self->{digest}),
                                                         );
        };
        my $ex = $@;
        if ($ex) {
            $rv->{completed_successfully} = 0;
            $rv->{failure_msg} = $ex->{error_message}; 
        }
        $rv->{configured_settings} = $self->{digest};
    }
    return $rv;
}

#----------------------------------------------------------------------

sub load_from_file {
    my($self, %params) = @_;
    my $conf_file_text;
    eval {
        $conf_file_text = read_file($self->{conf_file}->{fcdn_path});
    };
    my $ex = $@;
    if ($ex) {
        $ex=~ s/ at \S*? line \d+.$//;
        $self->{load_from_file_failure_msg} = "Cannot read configuration file '$self->{conf_file}->{logical_name}': $ex";
        $self->{logger}->error($self->{load_from_file_failure_msg});
        $self->{load_from_file_successfully} = 0;
        return;
    }
    eval {
        $self->{conf_file_content} = decode_json($conf_file_text);
    };
    $ex = $@;
    if ($ex) {
        $ex=~ s/ at \S*? line \d+.$//;
        $self->{load_from_file_failure_msg} = "Cannot parse content of configuration file '$self->{conf_file}->{logical_name}': $ex";
        $self->{logger}->error($self->{load_from_file_failure_msg});
        $self->{load_from_file_successfully} = 0;
        return;
    }
    $self->{load_from_file_successfully} = 1;
}

#----------------------------------------------------------------------

sub set_values {
    my($self, %params) = @_;
    $self->{seed} = $self->{cookie_protection_seed} || $self->{conf_file_content}->{wafSettings}->{seed};
    $self->{digest} = $self->{cookie_protection_digest} || $self->{conf_file_content}->{wafSettings}->{digest};
}

#----------------------------------------------------------------------

sub regenerate_data_protection_configuration {
    my($self, %params) = @_;
    my $data_protection = F5::ASMConfig::DataProtection->new(secure_key => F5::DataProtection::generate_key());
    $data_protection->{grace_accepting_interval} = 1;
    $self->{client}->update_data_protection($self->{client}->{session},
                                            F5::ASMConfig::CallFlags->new(),
                                            undef,
                                            $data_protection);
}

#----------------------------------------------------------------------

1;
